@@ -5,7 +5,7 @@ package JSON::Schema::Modern::Document::OpenAPI;
 # ABSTRACT: One OpenAPI v3.1 document
 # KEYWORDS: JSON Schema data validation request response OpenAPI
 
-our $VERSION = '0.046';
+our $VERSION = '0.047';
 
 use 5.020;
 use Moo;
@@ -25,6 +25,7 @@ use Ref::Util 'is_plain_hashref';
 use MooX::HandlesVia;
 use MooX::TypeTiny 0.002002;
 use Types::Standard qw(InstanceOf HashRef Str Enum);
+use Storable 'dclone';
 use namespace::clean;
 
 extends 'JSON::Schema::Modern::Document';
@@ -199,7 +200,7 @@ sub traverse ($self, $evaluator) {
   # are identical."
   my %seen_path;
   foreach my $path (sort keys $schema->{paths}->%*) {
-    my $normalized = $path =~ s/\{[^}]+\}/\x00/r;
+    my $normalized = $path =~ s/\{[^\}]+\}/\x00/r;
     if (my $first_path = $seen_path{$normalized}) {
       ()= E({ %$state, data_path => jsonp('/paths', $path),
         initial_schema_uri => Mojo::URL->new(DEFAULT_METASCHEMA) },
@@ -243,6 +244,25 @@ sub traverse ($self, $evaluator) {
   return $state;
 }
 
+sub recursive_get ($self, $json_pointer) {
+  my $base = $self->canonical_uri;
+  my $uri = Mojo::URL->new->fragment($json_pointer);
+  my ($depth, $schema);
+
+  while ($uri) {
+    die 'maximum evaluation depth exceeded' if $depth++ > $self->evaluator->max_traversal_depth;
+    $uri = Mojo::URL->new($uri)->to_abs($base);
+    my $schema_info = $self->evaluator->_fetch_from_uri($uri);
+    die('unable to find resource ', $uri) if not $schema_info;
+    $schema = $schema_info->{schema};
+    $base = $schema_info->{canonical_uri};
+    $uri = $schema->{'$ref'};
+  }
+
+  $schema = dclone($schema);
+  return wantarray ? ($schema, $base) : $schema;
+}
+
 ######## NO PUBLIC INTERFACES FOLLOW THIS POINT ########
 
 sub _add_vocab_and_default_schemas ($self) {
@@ -306,7 +326,7 @@ JSON::Schema::Modern::Document::OpenAPI - One OpenAPI v3.1 document
 
 =head1 VERSION
 
-version 0.046
+version 0.047
 
 =head1 SYNOPSIS
 
@@ -379,6 +399,12 @@ Returns the json pointer location of the operation containing the provided C<ope
 for passing to C<< $document->get(..) >>), or C<undef> if the location does not exist in the
 document.
 
+=head2 recursive_get
+
+Given a json pointer, get the schema at that location, resolving any C<$ref>s along the way.
+Returns the schema data in scalar context, or a tuple of the schema data and the canonical URI of
+the referenced location in list context.
+
 =head1 SEE ALSO
 
 =over 4
