@@ -1,11 +1,11 @@
 use strict;
 use warnings;
-package OpenAPI::Modern; # git description: v0.045-9-g275a796
+package OpenAPI::Modern; # git description: v0.046-13-g0e7dae9
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Validate HTTP requests and responses against an OpenAPI v3.1 document
 # KEYWORDS: validation evaluation JSON Schema OpenAPI v3.1 Swagger HTTP request response
 
-our $VERSION = '0.046';
+our $VERSION = '0.047';
 
 use 5.020;
 use utf8;
@@ -25,10 +25,9 @@ use Scalar::Util 'looks_like_number';
 use Feature::Compat::Try;
 use Encode 2.89;
 use URI::Escape ();
-use JSON::Schema::Modern 0.557;
+use JSON::Schema::Modern 0.560;
 use JSON::Schema::Modern::Utilities 0.531 qw(jsonp unjsonp canonical_uri E abort is_equal is_elements_unique);
 use JSON::Schema::Modern::Document::OpenAPI;
-use MooX::HandlesVia;
 use MooX::TypeTiny 0.002002;
 use Types::Standard 'InstanceOf';
 use constant { true => JSON::PP::true, false => JSON::PP::false };
@@ -83,6 +82,9 @@ around BUILDARGS => sub ($orig, $class, @args) {
 };
 
 sub validate_request ($self, $request, $options = {}) {
+  croak '$request and $options->{request} are inconsistent'
+    if $request and $options->{request} and $request != $options->{request};
+
   my $state = {
     data_path => '/request',
     initial_schema_uri => $self->openapi_uri,   # the canonical URI as of the start or last $id, or the last traversed $ref
@@ -93,8 +95,6 @@ sub validate_request ($self, $request, $options = {}) {
   };
 
   try {
-    croak '$request and $options->{request} are inconsistent'
-      if $request and $options->{request} and $request != $options->{request};
     $options->{request} //= $request;
     my $path_ok = $self->find_path($options);
     $request = $options->{request};   # now guaranteed to be a Mojo::Message::Request
@@ -153,9 +153,20 @@ sub validate_request ($self, $request, $options = {}) {
         if not exists $request_parameters_processed->{path}{$path_name};
     }
 
-    $state->{data_path} = jsonp($state->{data_path}, 'body');
     $state->{schema_path} = jsonp($state->{schema_path}, $method);
 
+    ()= E({ %$state, data_path => jsonp($state->{data_path}, 'header') },
+        'RFC9112 §6.2-2: A sender MUST NOT send a Content-Length header field in any message that contains a Transfer-Encoding header field')
+      if defined $request->headers->content_length and $request->content->is_chunked;
+
+    # RFC9112 §6.3-7: A user agent that sends a request that contains a message body MUST send
+    # either a valid Content-Length header field or use the chunked transfer coding.
+    ()= E({ %$state, data_path => jsonp($state->{data_path}, 'header') }, 'missing header: Content-Length')
+      if $request->body_size and not $request->headers->content_length
+        and not $request->content->is_chunked;
+
+    $state->{data_path} = jsonp($state->{data_path}, 'body');
+
     if (my $body_obj = $operation->{requestBody}) {
       $state->{schema_path} = jsonp($state->{schema_path}, 'requestBody');
 
@@ -163,7 +174,7 @@ sub validate_request ($self, $request, $options = {}) {
         $body_obj = $self->_resolve_ref('request-body', $ref, $state);
       }
 
-      if ($request->headers->content_length // $request->body_size) {
+      if ($request->body_size) {
         $self->_validate_body_content($state, $body_obj->{content}, $request);
       }
       elsif ($body_obj->{required}) {
@@ -171,6 +182,7 @@ sub validate_request ($self, $request, $options = {}) {
       }
     }
     else {
+      # we presume that no body specification for GET and HEAD requests -> no body is expected
       ()= E($state, 'unspecified body is present in %s request', uc $method)
         if ($method eq 'get' or $method eq 'head')
           and $request->headers->content_length // $request->body_size;
@@ -225,7 +237,7 @@ sub validate_response ($self, $response, $options = {}) {
     $response = _convert_response($response);   # now guaranteed to be a Mojo::Message::Response
 
     if (my $error = $response->error) {
-      ()= E($state, $response->error->{message});
+      ()= E($state, $error->{message});
       return $self->_result($state, 1);
     }
 
@@ -240,6 +252,10 @@ sub validate_response ($self, $response, $options = {}) {
         if $response->is_success and $method eq 'connect';
     }
 
+    ()= E({ %$state, data_path => jsonp($state->{data_path}, 'header') },
+        'RFC9112 §6.2-2: A sender MUST NOT send a Content-Length header field in any message that contains a Transfer-Encoding header field')
+      if defined $response->headers->content_length and $response->content->is_chunked;
+
     my $response_name = first { exists $operation->{responses}{$_} }
       $response->code, substr(sprintf('%03s', $response->code), 0, -2).'XX', 'default';
 
@@ -300,7 +316,7 @@ sub find_path ($self, $options) {
   };
 
   if ($options->{request} and my $error = $options->{request}->error) {
-    ()= E({ %$state, data_path => '/request' }, $options->{request}->error->{message});
+    ()= E({ %$state, data_path => '/request' }, $error->{message});
     return $self->_result($state, 1);
   }
 
@@ -476,6 +492,7 @@ sub _validate_header_parameter ($self, $state, $header_name, $header_obj, $heade
   return 1 if grep fc $header_name eq fc $_, qw(Accept Content-Type Authorization);
 
   # NOTE: for now, we will only support a single header value.
+  # TODO: explode multiple values into an array when type: array is requested.
   @$headers = map s/^\s*//r =~ s/\s*$//r, @$headers;
 
   if (not @$headers) {
@@ -519,7 +536,10 @@ sub _validate_parameter_content ($self, $state, $param_obj, $content_ref) {
     return $self->_evaluate_subschema($content_ref->$*, $schema, $state);
   }
 
-  $state = { %$state, schema_path => jsonp($state->{schema_path}, 'schema') };
+  # TODO: handle multi-valued elements like headers and query parameters, when type=array
+  # requested, and consider 'style' and 'explode' settings
+
+  $state = { %$state, schema_path => jsonp($state->{schema_path}, 'schema'), stringy_numbers => 1 };
   $self->_evaluate_subschema($content_ref->$*, $param_obj->{schema}, $state);
 }
 
@@ -644,18 +664,6 @@ sub _evaluate_subschema ($self, $data, $schema, $state) {
 
   return 1 if !keys(%$schema);  # schema is {}
 
-  # treat numeric-looking data as a string, unless "type" explicitly requests number or integer.
-  if (is_plain_hashref($schema) and exists $schema->{type} and not is_plain_arrayref($schema->{type})
-      and grep $schema->{type} eq $_, qw(number integer) and looks_like_number($data)) {
-    $data = $data+0;
-  }
-  elsif (defined $data and not is_ref($data)) {
-    $data = $data.'';
-  }
-
-  # TODO: also handle multi-valued elements like headers and query parameters, when type=array requested
-  # (and possibly coerce their numeric-looking elements as well)
-
   my $result = $self->evaluator->evaluate(
     $data, canonical_uri($state),
     {
@@ -663,6 +671,7 @@ sub _evaluate_subschema ($self, $data, $schema, $state) {
       traversed_schema_path => $state->{traversed_schema_path}.$state->{schema_path},
       effective_base_uri => $state->{effective_base_uri},
       collect_annotations => 1,
+      $state->{stringy_numbers} ? ( stringy_numbers => 1 ) : (),
     },
   );
 
@@ -716,7 +725,7 @@ OpenAPI::Modern - Validate HTTP requests and responses against an OpenAPI v3.1 d
 
 =head1 VERSION
 
-version 0.046
+version 0.047
 
 =head1 SYNOPSIS
 
@@ -879,6 +888,16 @@ The data structure describing the OpenAPI document. See L<the specification/http
 The L<JSON::Schema::Modern::Document::OpenAPI> document that holds the OpenAPI information to be
 used for validation.
 
+=head2 document_get
+
+  my $parameter_data = $openapi->document_get('/paths/~1foo~1{foo_id}/get/parameters/0');
+
+Fetches the subschema at the provided JSON pointer.
+Proxies to L<JSON::Schema::Modern::Document::OpenAPI/get>.
+This is not recursive (does not follow C<$ref> chains) -- for that, use
+C<< $openapi->openapi_document->recursive_get($json_pointer) >>, in
+L<JSON::Schema::Modern::Document::OpenAPI/recursive_get>.
+
 =head2 evaluator
 
 The L<JSON::Schema::Modern> object to use for all URI resolution and JSON Schema evaluation.
